from contextlib import contextmanager
from typing import Type

from django.db import connection, transaction, models
from psycopg2.sql import Identifier, SQL

__all__ = ['lock_table']


@contextmanager
def lock_table(model: Type[models.Model]):
    """Locks the table in SHARE ROW EXCLUSIVE MODE

    :param model:  The model whose table you want to lock.
    """
    with transaction.atomic():
        try:
            with connection.cursor() as cursor:
                sql = SQL('LOCK TABLE {tbl} IN SHARE ROW EXCLUSIVE MODE') \
                    .format(
                    tbl=Identifier(model._meta.db_table),
                )
                cursor.execute(sql)
                yield
        finally:
            pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['autosequence']

package_data = \
{'': ['*']}

install_requires = \
['django>=2.2']

setup_kwargs = {
    'name': 'django-autosequence',
    'version': '0.8.0',
    'description': 'A Django model field providing a configurable automatic sequence of values.',
    'long_description': None,
    'author': 'Andrew Cordery',
    'author_email': 'cordery@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/cordery/django-autosequence',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4',
}


setup(**setup_kwargs)
